@@ -1,4 +1,4 @@
-NUM_VERSION = (2011, 2)
+NUM_VERSION = (2011, 3)
 VERSION = ".".join(str(nv) for nv in NUM_VERSION)
 
 
@@ -61,7 +61,7 @@ def runscript(mainpyfile, args=None, pre_run="", steal_output=False):
         try:
             dbg._runscript(mainpyfile)
         except SystemExit, se:
-            status_msg = "The debuggee exited normally with status code was %d.\n\n" % se.code
+            status_msg = "The debuggee exited normally with status code %s.\n\n" % se.code
         except:
             dbg.post_mortem = True
             dbg.interaction(None, sys.exc_info())
@@ -80,7 +80,7 @@ def runscript(mainpyfile, args=None, pre_run="", steal_output=False):
                 % status_msg),
                 urwid.Text("\n\nIf you decide to restart, this command will be run prior to "
                 "actually restarting:"),
-                urwid.AttrWrap(pre_run_edit, "value")
+                urwid.AttrMap(pre_run_edit, "value")
                 ]),
             [
                 ("Restart", "restart"),
