@@ -5,7 +5,7 @@ from __future__ import division
 import urwid
 import bdb
 
-
+from pudb import CONFIG
 
 
 HELP_TEXT = """\
@@ -42,19 +42,24 @@ Keys:
     V - focus variables
     S - focus stack
     B - focus breakpoint list
-    +/- - grow/shrink sidebar
-    _/= - minimize/maximize sidebar
 
     f1/?/H - show this help screen
     q - quit
 
+Side-bar related:
+
+    +/- - grow/shrink sidebar
+    _/= - minimize/maximize sidebar
+    [/] - grow/shrink relative size of active sidebar box
+
 Keys in variables list:
 
     \ - expand/collapse
-    t/r/s - show type/repr/str for this variable
+    t/r/s/c - show type/repr/str/custom for this variable
     h - toggle highlighting
     @ - toggle repetition at top
     * - toggle private members
+    w - toggle line wrapping
     n/insert - add new watch expression
     enter - edit options (also to delete)
 
@@ -98,7 +103,8 @@ OTHER DEALINGS IN THE SOFTWARE.
 
 
 
-# debugger interface ----------------------------------------------------------
+# {{{ debugger interface
+
 class Debugger(bdb.Bdb):
     def __init__(self, steal_output=False):
         bdb.Bdb.__init__(self)
@@ -149,6 +155,8 @@ class Debugger(bdb.Bdb):
         self.ui.update_var_view()
         self.ui.update_stack()
 
+        self.ui.stack_list._w.set_focus(self.ui.translate_ui_stack_index(index))
+
     def move_up_frame(self):
         if self.curindex > 0:
             self.set_frame_index(self.curindex-1)
@@ -267,6 +275,8 @@ class Debugger(bdb.Bdb):
         statement = 'execfile( "%s")' % filename
         self.run(statement, globals=globals_, locals=locals_)
 
+# }}}
+
 
 
 
@@ -281,22 +291,26 @@ from pudb.var_view import FrameVarInfoKeeper
 
 
 class DebuggerUI(FrameVarInfoKeeper):
+    # {{{ constructor
+
     def __init__(self, dbg):
         FrameVarInfoKeeper.__init__(self)
 
         self.debugger = dbg
-        Attr = urwid.AttrWrap
+        Attr = urwid.AttrMap
 
         self.search_box = None
         self.last_module_filter = ""
 
+        # {{{ build ui
+
         self.source = urwid.SimpleListWalker([])
         self.source_list = urwid.ListBox(self.source)
         self.source_sigwrap = SignalWrap(self.source_list)
         self.source_hscroll_start = 0
 
         self.lhs_col = urwid.Pile([
-            ("weight", 1, urwid.AttrWrap(self.source_sigwrap, "source"))
+            ("weight", 1, urwid.AttrMap(self.source_sigwrap, "source"))
             ])
 
         self.locals = urwid.SimpleListWalker([])
@@ -312,34 +326,56 @@ class DebuggerUI(FrameVarInfoKeeper):
                 urwid.ListBox(self.bp_walker))
 
         self.rhs_col = urwid.Pile([
-            Attr(urwid.Pile([
+            ("weight", float(CONFIG["variables_weight"]), Attr(urwid.Pile([
                 ("flow", urwid.Text(make_hotkey_markup("_Variables:"))),
                 Attr(self.var_list, "variables"),
-                ]), None, "focused sidebar"),
-            Attr(urwid.Pile([
+                ]), None, "focused sidebar"),),
+            ("weight", float(CONFIG["stack_weight"]), Attr(urwid.Pile([
                 ("flow", urwid.Text(make_hotkey_markup("_Stack:"))),
                 Attr(self.stack_list, "stack"),
-                ]), None, "focused sidebar"),
-            Attr(urwid.Pile([
+                ]), None, "focused sidebar"),),
+            ("weight", float(CONFIG["breakpoints_weight"]), Attr(urwid.Pile([
                 ("flow", urwid.Text(make_hotkey_markup("_Breakpoints:"))),
                 Attr(self.bp_list, "breakpoint"),
-                ]), None, "focused sidebar"),
+                ]), None, "focused sidebar"),),
             ])
 
         self.columns = urwid.Columns(
                     [
                         ("weight", 1, self.lhs_col),
-                        ("weight", 0.5, self.rhs_col),
+                        ("weight", float(CONFIG["sidebar_width"]), self.rhs_col),
                         ],
                     dividechars=1)
 
         self.caption = urwid.Text("")
-        header = urwid.AttrWrap(self.caption, "header")
+        header = urwid.AttrMap(self.caption, "header")
         self.top = SignalWrap(urwid.Frame(
-            urwid.AttrWrap(self.columns, "background"),
+            urwid.AttrMap(self.columns, "background"),
             header))
 
-        # variable listeners --------------------------------------------------
+        # }}}
+
+        from functools import partial
+
+        def change_rhs_box(name, index, direction, w, size, key):
+            from pudb.settings import save_config
+
+            _, weight = self.rhs_col.item_types[index]
+
+            if direction < 0:
+                if weight > 1/5:
+                    weight /= 1.25
+            else:
+                if weight < 5:
+                    weight *= 1.25
+
+            CONFIG[name+"_weight"] = weight
+            save_config(CONFIG)
+            self.rhs_col.item_types[index] = "weight", weight
+            self.rhs_col._invalidate()
+
+
+        # {{{ variables listeners
         def change_var_state(w, size, key):
             var, pos = self.var_list._w.get_focus()
 
@@ -350,9 +386,11 @@ class DebuggerUI(FrameVarInfoKeeper):
             elif key == "t": iinfo.display_type = "type"
             elif key == "r": iinfo.display_type = "repr"
             elif key == "s": iinfo.display_type = "str"
+            elif key == "c": iinfo.display_type = CONFIG["custom_stringifier"]
             elif key == "h": iinfo.highlighted = not iinfo.highlighted
             elif key == "@": iinfo.repeated_at_top = not iinfo.repeated_at_top
             elif key == "*": iinfo.show_private_members = not iinfo.show_private_members
+            elif key == "w": iinfo.wrap = not iinfo.wrap
 
             self.update_var_view()
 
@@ -374,7 +412,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                 watch_edit = urwid.Edit([
                     ("label", "Watch expression: ")
                     ], var.watch_expr.expression)
-                id_segment = [urwid.AttrWrap(watch_edit, "value"), urwid.Text("")]
+                id_segment = [urwid.AttrMap(watch_edit, "value"), urwid.Text("")]
 
                 buttons.extend([None, ("Delete", "del")])
 
@@ -394,7 +432,11 @@ class DebuggerUI(FrameVarInfoKeeper):
                     iinfo.display_type == "repr")
             rb_show_str = urwid.RadioButton(rb_grp, "Show str()",
                     iinfo.display_type == "str")
+            rb_show_custom = urwid.RadioButton(rb_grp, "Show custom (set in prefs)",
+                    iinfo.display_type == CONFIG["custom_stringifier"])
 
+
+            wrap_checkbox = urwid.CheckBox("Line Wrap", iinfo.wrap)
             expanded_checkbox = urwid.CheckBox("Expanded", iinfo.show_detail)
             highlighted_checkbox = urwid.CheckBox("Highlighted", iinfo.highlighted)
             repeated_at_top_checkbox = urwid.CheckBox("Repeated at top", iinfo.repeated_at_top)
@@ -404,6 +446,7 @@ class DebuggerUI(FrameVarInfoKeeper):
             lb = urwid.ListBox(
                 id_segment+rb_grp+[
                 urwid.Text(""),
+                wrap_checkbox,
                 expanded_checkbox,
                 highlighted_checkbox,
                 repeated_at_top_checkbox,
@@ -414,13 +457,19 @@ class DebuggerUI(FrameVarInfoKeeper):
 
             if result == True:
                 iinfo.show_detail = expanded_checkbox.get_state()
+                iinfo.wrap = wrap_checkbox.get_state()
                 iinfo.highlighted = highlighted_checkbox.get_state()
                 iinfo.repeated_at_top = repeated_at_top_checkbox.get_state()
                 iinfo.show_private_members = show_private_checkbox.get_state()
 
-                if rb_show_type.get_state(): iinfo.display_type = "type"
-                elif rb_show_repr.get_state(): iinfo.display_type = "repr"
-                elif rb_show_str.get_state(): iinfo.display_type = "str"
+                if rb_show_type.get_state():
+                    iinfo.display_type = "type"
+                elif rb_show_repr.get_state():
+                    iinfo.display_type = "repr"
+                elif rb_show_str.get_state():
+                    iinfo.display_type = "str"
+                elif rb_show_custom.get_state():
+                    iinfo.display_type = CONFIG["custom_stringifier"]
 
                 if var.watch_expr is not None:
                     var.watch_expr.expression = watch_edit.get_edit_text()
@@ -439,7 +488,7 @@ class DebuggerUI(FrameVarInfoKeeper):
 
             if self.dialog(
                     urwid.ListBox([
-                        urwid.AttrWrap(watch_edit, "value")
+                        urwid.AttrMap(watch_edit, "value")
                         ]),
                     [
                         ("OK", True),
@@ -456,44 +505,59 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.var_list.listen("t", change_var_state)
         self.var_list.listen("r", change_var_state)
         self.var_list.listen("s", change_var_state)
+        self.var_list.listen("c", change_var_state)
         self.var_list.listen("h", change_var_state)
         self.var_list.listen("@", change_var_state)
+        self.var_list.listen("*", change_var_state)
+        self.var_list.listen("w", change_var_state)
         self.var_list.listen("enter", edit_inspector_detail)
         self.var_list.listen("n", insert_watch)
         self.var_list.listen("insert", insert_watch)
 
-        # stack listeners -----------------------------------------------------
+        self.var_list.listen("[", partial(change_rhs_box, 'variables', 0, -1))
+        self.var_list.listen("]", partial(change_rhs_box, 'variables', 0, 1))
+
+        # }}}
+
+        # {{{ stack listeners
         def examine_frame(w, size, key):
+            from pudb import CONFIG
             _, pos = self.stack_list._w.get_focus()
-            self.debugger.set_frame_index(len(self.debugger.stack)-1-pos)
+            self.debugger.set_frame_index(self.translate_ui_stack_index(pos))
 
         self.stack_list.listen("enter", examine_frame)
 
         def move_stack_up(w, size, key):
             self.debugger.move_up_frame()
+
         def move_stack_down(w, size, key):
             self.debugger.move_down_frame()
 
         self.stack_list.listen("u", move_stack_up)
         self.stack_list.listen("d", move_stack_down)
-        self.source_sigwrap.listen("u", move_stack_up)
-        self.source_sigwrap.listen("d", move_stack_down)
 
-        # stack listeners -----------------------------------------------------
+        self.stack_list.listen("[", partial(change_rhs_box, 'stack', 1, -1))
+        self.stack_list.listen("]", partial(change_rhs_box, 'stack', 1, 1))
+
+        # }}}
+
+        # {{{ breakpoint listeners
         def save_breakpoints(w, size, key):
             self.debugger.save_breakpoints()
 
         def delete_breakpoint(w, size, key):
-            _, pos = self.bp_list._w.get_focus()
-            bp = self._get_bp_list()[pos]
-            if self.shown_file == bp.file:
-                self.source[bp.line-1].set_breakpoint(False)
+            bp_list = self._get_bp_list()
+            if bp_list:
+                _, pos = self.bp_list._w.get_focus()
+                bp = bp_list[pos]
+                if self.shown_file == bp.file:
+                    self.source[bp.line-1].set_breakpoint(False)
 
-            err = self.debugger.clear_break(bp.file, bp.line)
-            if err:
-                self.message("Error clearing breakpoint:\n"+ err)
-            else:
-                self.update_breakpoints()
+                err = self.debugger.clear_break(bp.file, bp.line)
+                if err:
+                    self.message("Error clearing breakpoint:\n"+ err)
+                else:
+                    self.update_breakpoints()
 
         def examine_breakpoint(w, size, key):
             bp_entry, pos = self.bp_list._w.get_focus()
@@ -523,8 +587,8 @@ class DebuggerUI(FrameVarInfoKeeper):
                 labelled_value("Hits: ", bp.hits),
                 urwid.Text(""),
                 enabled_checkbox,
-                urwid.AttrWrap(cond_edit, "value", "value"),
-                urwid.AttrWrap(ign_count_edit, "value", "value"),
+                urwid.AttrMap(cond_edit, "value", "value"),
+                urwid.AttrMap(ign_count_edit, "value", "value"),
                 ])
 
             result = self.dialog(lb, [
@@ -560,7 +624,13 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.bp_list.listen("d", delete_breakpoint)
         self.bp_list.listen("s", save_breakpoints)
 
-        # top-level listeners -------------------------------------------------
+        self.bp_list.listen("[", partial(change_rhs_box, 'breakpoints', 2, -1))
+        self.bp_list.listen("]", partial(change_rhs_box, 'breakpoints', 2, 1))
+
+        # }}}
+
+        # {{{ source listeners
+
         def end():
             self.debugger.save_breakpoints()
             self.quit_event_loop = True
@@ -634,7 +704,7 @@ class DebuggerUI(FrameVarInfoKeeper):
             if self.dialog(
                     urwid.ListBox([
                         labelled_value("File :", self.shown_file),
-                        urwid.AttrWrap(lineno_edit, "value")
+                        urwid.AttrMap(lineno_edit, "value")
                         ]),
                     [
                         ("OK", True),
@@ -674,18 +744,18 @@ class DebuggerUI(FrameVarInfoKeeper):
                 _, search_start = self.source.get_focus()
 
                 self.search_box = SearchBox(self)
-                self.search_attrwrap = urwid.AttrWrap(
+                self.search_AttrMap = urwid.AttrMap(
                         self.search_box, "search box")
 
                 self.lhs_col.item_types.insert(
                         0, ("flow", None))
-                self.lhs_col.widget_list.insert( 0, self.search_attrwrap)
+                self.lhs_col.widget_list.insert( 0, self.search_AttrMap)
 
                 self.columns.set_focus(self.lhs_col)
-                self.lhs_col.set_focus(self.search_attrwrap)
+                self.lhs_col.set_focus(self.search_AttrMap)
             else:
                 self.columns.set_focus(self.lhs_col)
-                self.lhs_col.set_focus(self.search_attrwrap)
+                self.lhs_col.set_focus(self.search_AttrMap)
                 self.search_box.restart_search()
 
         def search_next(w, size, key):
@@ -762,7 +832,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                     return ext == ".py"
 
             new_mod_text = SelectableText("-- update me --")
-            new_mod_entry = urwid.AttrWrap(new_mod_text,
+            new_mod_entry = urwid.AttrMap(new_mod_text,
                     None, "focused selectable")
 
             def build_filtered_mod_list(filt_string=""):
@@ -770,7 +840,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                         for name, mod in sys.modules.items()
                         if mod_exists(mod))
 
-                result = [urwid.AttrWrap(SelectableText(mod),
+                result = [urwid.AttrMap(SelectableText(mod),
                         None, "focused selectable")
                         for mod in modules if filt_string in mod]
                 new_mod_text.set_text("<<< IMPORT MODULE '%s' >>>" % filt_string)
@@ -806,9 +876,9 @@ class DebuggerUI(FrameVarInfoKeeper):
             lb = urwid.ListBox(mod_list)
 
             w = urwid.Pile([
-                ("flow", urwid.AttrWrap(filt_edit, "value")),
+                ("flow", urwid.AttrMap(filt_edit, "value")),
                 ("fixed", 1, urwid.SolidFill()),
-                urwid.AttrWrap(lb, "selectable")])
+                urwid.AttrMap(lb, "selectable")])
 
             while True:
                 result = self.dialog(w, [
@@ -836,14 +906,14 @@ class DebuggerUI(FrameVarInfoKeeper):
                             show_mod(sys.modules[str(new_mod_name)])
                             break
                     else:
-                        show_mod(sys.modules[widget.get_text()[0]])
+                        show_mod(sys.modules[widget.base_widget.get_text()[0]])
                         break
                 elif result == False:
                     break
                 elif result == "reload":
                     widget, pos = lb.get_focus()
                     if widget is not new_mod_entry:
-                        mod_name = widget.get_text()[0]
+                        mod_name = widget.base_widget.get_text()[0]
                         mod = sys.modules[mod_name]
                         reload(mod)
                         self.message("'%s' was successfully reloaded." % mod_name)
@@ -882,7 +952,13 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.source_sigwrap.listen("b", toggle_breakpoint)
         self.source_sigwrap.listen("m", pick_module)
 
-        # top-level listeners -------------------------------------------------
+        self.source_sigwrap.listen("u", move_stack_up)
+        self.source_sigwrap.listen("d", move_stack_down)
+
+        # }}}
+
+        # {{{ top-level listeners
+
         def show_output(w, size, key):
             self.screen.stop()
             raw_input("Hit Enter to return:")
@@ -921,7 +997,6 @@ class DebuggerUI(FrameVarInfoKeeper):
 
             curframe = self.debugger.curframe
 
-            from pudb import CONFIG
             import pudb.shell as shell
             if shell.HAVE_IPYTHON and CONFIG["shell"] == "ipython":
                 runner = shell.run_ipython_shell
@@ -944,26 +1019,46 @@ class DebuggerUI(FrameVarInfoKeeper):
                 self.rhs_col.set_focus(self.rhs_col.widget_list[subself.idx])
 
         def max_sidebar(w, size, key):
-            self.columns.column_types[1] = "weight", 5
+            from pudb.settings import save_config
+
+            weight = 5
+            CONFIG["sidebar_width"] = weight
+            save_config(CONFIG)
+
+            self.columns.column_types[1] = "weight", weight
             self.columns._invalidate()
 
         def min_sidebar(w, size, key):
-            self.columns.column_types[1] = "weight", 1/5
+            from pudb.settings import save_config
+
+            weight = 1/5
+            CONFIG["sidebar_width"] = weight
+            save_config(CONFIG)
+
+            self.columns.column_types[1] = "weight", weight
             self.columns._invalidate()
 
         def grow_sidebar(w, size, key):
+            from pudb.settings import save_config
+
             _, weight = self.columns.column_types[1]
 
             if weight < 5:
                 weight *= 1.25
+                CONFIG["sidebar_width"] = weight
+                save_config(CONFIG)
                 self.columns.column_types[1] = "weight", weight
                 self.columns._invalidate()
 
         def shrink_sidebar(w, size, key):
+            from pudb.settings import save_config
+
             _, weight = self.columns.column_types[1]
 
             if weight > 1/5:
                 weight /= 1.25
+                CONFIG["sidebar_width"] = weight
+                save_config(CONFIG)
                 self.columns.column_types[1] = "weight", weight
                 self.columns._invalidate()
 
@@ -972,14 +1067,7 @@ class DebuggerUI(FrameVarInfoKeeper):
             end()
 
         def do_edit_config(w, size, key):
-            from pudb.settings import edit_config, save_config
-            from pudb import CONFIG
-            edit_config(self, CONFIG)
-            save_config(CONFIG)
-            self.setup_palette(self.screen)
-
-            for sl in self.source:
-                sl._invalidate()
+            self.run_edit_config()
 
         def help(w, size, key):
             self.message(HELP_TEXT, title="PuDB Help")
@@ -1003,7 +1091,10 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.top.listen("f1", help)
         self.top.listen("?", help)
 
-        # setup ---------------------------------------------------------------
+        # }}}
+
+        # {{{ setup
+
         import urwid.raw_display as display
 
         self.screen = display.Screen()
@@ -1016,11 +1107,32 @@ class DebuggerUI(FrameVarInfoKeeper):
 
         self.quit_event_loop = False
 
+        # }}}
+
+    # }}}
+
+    # {{{ UI helpers
+
+    def translate_ui_stack_index(self, index):
+        # note: self-inverse
+
+        if CONFIG["current_stack_frame"] == "top":
+            return len(self.debugger.stack)-1-index
+        elif CONFIG["current_stack_frame"] == "bottom":
+            return index
+        else:
+            raise ValueError("invalid value for 'current_stack_frame' pref")
+
     def message(self, msg, title="Message", **kwargs):
         self.call_with_ui(self.dialog,
                 urwid.ListBox([urwid.Text(msg)]),
                 [("OK", True)], title=title, **kwargs)
 
+    def run_edit_config(self):
+        from pudb.settings import edit_config, save_config
+        edit_config(self, CONFIG)
+        save_config(CONFIG)
+
     def dialog(self, content, buttons_and_results,
             title=None, bind_enter_esc=True, focus_buttons=False,
             extra_bindings=[]):
@@ -1031,7 +1143,7 @@ class DebuggerUI(FrameVarInfoKeeper):
             def __call__(subself, btn):
                 self.quit_event_loop = [subself.res]
 
-        Attr = urwid.AttrWrap
+        Attr = urwid.AttrMap
 
         if bind_enter_esc:
             content = SignalWrap(content)
@@ -1060,7 +1172,7 @@ class DebuggerUI(FrameVarInfoKeeper):
 
         if title is not None:
             w = urwid.Pile([
-                ("flow", urwid.AttrWrap(
+                ("flow", urwid.AttrMap(
                     urwid.Text(title, align="center"),
                     "dialog title")),
                 ("fixed", 1, urwid.SolidFill()),
@@ -1088,12 +1200,14 @@ class DebuggerUI(FrameVarInfoKeeper):
         may_use_fancy_formats = isinstance(screen, RawScreen) and \
                 not hasattr(urwid.escape, "_fg_attr_xterm")
 
-        from pudb import CONFIG
         from pudb.theme import get_palette
         screen.register_palette(
                 get_palette(may_use_fancy_formats, CONFIG["theme"]))
 
-    # UI enter/exit -----------------------------------------------------------
+    # }}}
+
+    # {{{ UI enter/exit
+
     def show(self):
         if self.show_count == 0:
             self.screen.start()
@@ -1111,7 +1225,9 @@ class DebuggerUI(FrameVarInfoKeeper):
         finally:
             self.hide()
 
-    # interaction -------------------------------------------------------------
+    # }}}
+
+    # {{{ interaction
     def event_loop(self, toplevel=None):
         prev_quit_loop = self.quit_event_loop
 
@@ -1124,7 +1240,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                         "Syntax highlighting disabled.")
 
         from pudb import CONFIG
-        WELCOME_LEVEL = "d"
+        WELCOME_LEVEL = "e002"
         if CONFIG["seen_welcome"] < WELCOME_LEVEL:
             CONFIG["seen_welcome"] = WELCOME_LEVEL
             from pudb import VERSION
@@ -1136,22 +1252,31 @@ class DebuggerUI(FrameVarInfoKeeper):
                     "a terminal. If you've worked with the excellent (but nowadays "
                     "ancient) DOS-based Turbo Pascal or C tools, PuDB's UI might "
                     "look familiar.\n\n"
-                    "New features in version 2011.2:\n\n"
-                    "- Fix for post-mortem debugging (submitted by 'Sundance')\n\n"
-                    "New features in version 2011.1:\n\n"
+                    "If you're new here, welcome! The help screen (invoked by hitting "
+                    "'?' after this message) should get you on your way.\n"
+                    "\nNew features in version 2011.3:\n\n"
+                    "- Finer-grained string highlighting (submitted by Aaron Meurer)\n"
+                    "- Prefs tweaks, instant-apply, top-down stack (submitted by Aaron Meurer)\n"
+                    "- Size changes in sidebar boxes (submitted by Aaron Meurer)\n"
+                    "- New theme 'midnight' (submitted by Aaron Meurer)\n"
+                    "- Support for IPython 0.11 (submitted by Chris Farrow)\n"
+                    "- Suport for custom stringifiers (submitted by Aaron Meurer)\n"
+                    "- Line wrapping in variables view (submitted by Aaron Meurer)\n"
+                    "\nNew features in version 2011.2:\n\n"
+                    "- Fix for post-mortem debugging (submitted by 'Sundance')\n"
+                    "\nNew features in version 2011.1:\n\n"
                     "- Breakpoints saved between sessions\n"
                     "- A new 'dark vim' theme\n"
-                    "(both contributed by Naveen Michaud-Agrawal)\n\n"
-                    "New features in version 0.93:\n\n"
+                    "(both contributed by Naveen Michaud-Agrawal)\n"
+                    "\nNew features in version 0.93:\n\n"
                     "- Stored preferences (no more pesky IPython prompt!)\n"
                     "- Themes\n"
                     "- Line numbers (optional)\n"
-                    "\nHit Ctrl-P to set up PuDB.\n\n"
-                    "If you're new here, welcome! The help screen (invoked by hitting "
-                    "'?' after this message) should get you on your way." % VERSION)
-
+                    % VERSION)
             from pudb.settings import save_config
             save_config(CONFIG)
+            self.run_edit_config()
+
 
         try:
             if toplevel is None:
@@ -1176,13 +1301,16 @@ class DebuggerUI(FrameVarInfoKeeper):
         finally:
             self.quit_event_loop = prev_quit_loop
 
-    # debugger-facing interface -----------------------------------------------
+    # }}}
+
+    # {{{ debugger-facing interface
+
     def interaction(self, exc_tuple):
         self.current_exc_tuple = exc_tuple
 
         from pudb import VERSION
         caption = [(None,
-            u"PuDB %s - ?:help  n:next  s:step into  b:breakpoint  o:output "
+            u"PuDB %s - ?:help  n:next  s:step into  b:breakpoint  o:output  "
             "t:run to cursor  !:python shell"
             % VERSION)]
 
@@ -1320,8 +1448,17 @@ class DebuggerUI(FrameVarInfoKeeper):
                     code.co_name, class_name,
                     self._format_fname(code.co_filename), lineno)
 
-        self.stack_walker[:] = [make_frame_ui(fl)
-                for fl in self.debugger.stack[::-1]]
+        from pudb import CONFIG
+
+        frame_uis = [make_frame_ui(fl) for fl in self.debugger.stack]
+        if CONFIG["current_stack_frame"] == "top":
+            frame_uis = frame_uis[::-1]
+        elif CONFIG["current_stack_frame"] == "bottom":
+            pass
+        else:
+            raise ValueError("invalid value for 'current_stack_frame' pref")
+
+        self.stack_walker[:] = frame_uis
 
     def show_exception(self, exc_type, exc_value, traceback):
         from traceback import format_exception
@@ -1330,3 +1467,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                 "".join(format_exception(
                     exc_type, exc_value, traceback)),
                 title="Exception Occurred")
+
+    # }}}
+
+# vim: foldmethod=marker
